@@ -40,8 +40,8 @@ char copyright[] =
 /*
  * From: @(#)ping.c	5.9 (Berkeley) 5/12/91
  */
-char rcsid[] = "$Id: ping.c,v 1.22 1997/06/08 19:39:47 dholland Exp $";
-char pkg[] = "netkit-base-0.10";
+char rcsid[] = "$Id: ping.c,v 1.33 1999/09/14 14:02:46 dholland Exp $";
+#include "../version.h"
 
 /*
  *			P I N G . C
@@ -65,7 +65,6 @@ char pkg[] = "netkit-base-0.10";
 #include <sys/socket.h>
 #include <sys/file.h>
 #include <sys/time.h>
-#include <sys/times.h>
 #include <sys/signal.h>
 
 #include <netinet/in.h>
@@ -92,37 +91,8 @@ char pkg[] = "netkit-base-0.10";
 
 #if defined(__GLIBC__) && (__GLIBC__ >= 2)
 #define icmphdr			icmp
-#ifdef NEED_ICMP_HEADERS
-#define ICMP_DEST_UNREACH	ICMP_UNREACH
-#define ICMP_NET_UNREACH	ICMP_UNREACH_NET
-#define ICMP_HOST_UNREACH	ICMP_UNREACH_HOST
-#define ICMP_PORT_UNREACH	ICMP_UNREACH_PORT
-#define ICMP_PROT_UNREACH	ICMP_UNREACH_PROTOCOL
-#define ICMP_FRAG_NEEDED	ICMP_UNREACH_NEEDFRAG
-#define ICMP_SR_FAILED		ICMP_UNREACH_SRCFAIL
-#define ICMP_NET_UNKNOWN	ICMP_UNREACH_NET_UNKNOWN
-#define ICMP_HOST_UNKNOWN	ICMP_UNREACH_HOST_UNKNOWN
-#define ICMP_HOST_ISOLATED	ICMP_UNREACH_ISOLATED
-#define ICMP_NET_UNR_TOS	ICMP_UNREACH_TOSNET
-#define ICMP_HOST_UNR_TOS	ICMP_UNREACH_TOSHOST
-#define ICMP_SOURCE_QUENCH	ICMP_SOURCEQUENCH
-#define ICMP_REDIR_NET		ICMP_REDIRECT_NET
-#define ICMP_REDIR_HOST		ICMP_REDIRECT_HOST
-#define ICMP_REDIR_NETTOS	ICMP_REDIRECT_TOSNET
-#define ICMP_REDIR_HOSTTOS	ICMP_REDIRECT_TOSHOST
-#define ICMP_TIME_EXCEEDED	ICMP_TIMXCEED
-#define ICMP_EXC_TTL		ICMP_TIMXCEED_INTRANS
-#define ICMP_EXC_FRAGTIME	ICMP_TIMXCEED_REASS
-#define	ICMP_PARAMETERPROB	ICMP_PARAMPROB
-#define ICMP_TIMESTAMP		ICMP_TSTAMP
-#define ICMP_TIMESTAMPREPLY	ICMP_TSTAMPREPLY
-#define ICMP_INFO_REQUEST	ICMP_IREQ
-#define ICMP_INFO_REPLY		ICMP_IREQREPLY
-#endif
 #else
-# ifndef ICMP_MINLEN
-# define ICMP_MINLEN	28
-# endif
+#define ICMP_MINLEN	28
 #define inet_ntoa(x) inet_ntoa(*((struct in_addr *)&(x)))
 #endif
 
@@ -167,7 +137,7 @@ int moptions;
 int mx_dup_ck = MAX_DUP_CHK;
 char rcvd_tbl[MAX_DUP_CHK / 8];
 
-struct sockaddr whereto;	/* who to ping */
+struct sockaddr_in whereto;	/* who to ping */
 int datalen = DEFDATALEN;
 int s;				/* socket file descriptor */
 u_char outpack[MAXPACKET];
@@ -175,13 +145,7 @@ char BSPACE = '\b';		/* characters written for flood */
 char DOT = '.';
 static char *hostname;
 static int ident;		/* process id to identify our packets */
-
-#ifdef timestamp_check
-#define MAX_TIME_CHK 32
-#define tvsame(left,right)  (((left).tv_sec == (right).tv_sec) && \
-				((left).tv_usec == (right).tv_usec))
-static struct timeval timestamp[MAX_TIME_CHK];
-#endif
+static time_t last_time;        /* prevents flood pinging via SIGALRM */
 
 /* counters */
 static long npackets;		/* max packets to transmit */
@@ -189,12 +153,11 @@ static long nreceived;		/* # of packets we got back */
 static long nrepeats;		/* number of duplicates */
 static long ntransmitted;	/* sequence # for outbound packets = #sent */
 static int interval = 1;	/* interval between packets */
-static int floodok = 1;         /* okay to send next flood ping? */
 
 /* timing */
 static int timing;		/* flag to do timing */
-static u_long tmin = ULONG_MAX;	/* minimum round trip time */
-static u_long tmax = 0;		/* maximum round trip time */
+static long tmin = LONG_MAX;	/* minimum round trip time */
+static long tmax = 0;		/* maximum round trip time */
 static u_long tsum;		/* sum of all times, for doing average */
 
 /* protos */
@@ -205,7 +168,7 @@ static void finish(int ignore);
 static void pinger(void);
 static void fill(void *bp, char *patp);
 static void usage(void);
-static void pr_pack(char *buf, int cc, struct sockaddr_in *from);
+static int pr_pack(char *buf, int cc, struct sockaddr_in *from);
 static void tvsub(struct timeval *out, struct timeval *in);
 static void pr_icmph(struct icmphdr *icp);
 static void pr_retip(struct iphdr *ip);
@@ -233,6 +196,10 @@ main(int argc, char *argv[])
 	__environ = &null;
 	am_i_root = (getuid()==0);
 
+	/* setup last_time to be current time -1 second */
+	time(&last_time);
+	last_time--;
+
 	/*
 	 * Pull this stuff up front so we can drop root if desired.
 	 */
@@ -248,6 +215,11 @@ main(int argc, char *argv[])
 		exit(2);
 	}
 
+	if (s==STDIN_FILENO || s==STDOUT_FILENO || s==STDERR_FILENO) {
+		/* would be nice to issue an error message, but to where? */
+		exit(255);
+	}
+
 #ifdef SAFE_TO_DROP_ROOT
 	setuid(getuid());
 #endif
@@ -317,7 +289,7 @@ main(int argc, char *argv[])
 		case 's':		/* size of packet to send */
 			if (!am_i_root) {
 				(void)fprintf(stderr,
-					"ping: %s\n", strerror(EPERM));
+				    "ping: %s\n", strerror(EPERM));
 				exit(2);
 			}
 			datalen = atoi(optarg);
@@ -374,8 +346,8 @@ main(int argc, char *argv[])
 		usage();
 	target = *argv;
 
-	memset(&whereto, 0, sizeof(struct sockaddr));
-	to = (struct sockaddr_in *)&whereto;
+	memset(&whereto, 0, sizeof(whereto));
+	to = &whereto;
 	to->sin_family = AF_INET;
 	if (inet_aton(target, &to->sin_addr)) {
 		hostname = target;
@@ -392,7 +364,8 @@ main(int argc, char *argv[])
 			hp->h_length = sizeof(to->sin_addr);
 		}
 		memcpy(&to->sin_addr, hp->h_addr, hp->h_length);
-		(void)strncpy(hnamebuf, hp->h_name, sizeof(hnamebuf) - 1);
+		strncpy(hnamebuf, hp->h_name, sizeof(hnamebuf) - 1);
+		hnamebuf[sizeof(hnamebuf)-1] = 0;
 		hostname = hnamebuf;
 	}
 
@@ -411,7 +384,7 @@ main(int argc, char *argv[])
 		exit(2);
 	}
 	if (!(options & F_PINGFILLED))
-		for (i = 8; i < datalen; ++i)
+		for (i = sizeof(struct timeval); i < datalen; ++i)
 			*datap++ = i;
 
 	ident = getpid() & 0xFFFF;
@@ -453,7 +426,7 @@ main(int argc, char *argv[])
 	 * ethernet, or just want to fill the arp cache to get some stuff for
 	 * /etc/ethers.
 	 */
-	hold = 48 * 1024;
+	hold = 64 * 1024;
 	(void)setsockopt(s, SOL_SOCKET, SO_RCVBUF, (char *)&hold,
 	    sizeof(hold));
 
@@ -499,20 +472,18 @@ main(int argc, char *argv[])
 
 	for (;;) {
 		struct sockaddr_in from;
+		socklen_t fromlen;
 		register int cc;
-		size_t fromlen;
+		static int got=1;
 
 		if (options & F_FLOOD) {
-			if ( floodok ) {
-				floodok = 0;
-				pinger();
-			}
+			if (got) pinger();
 			timeout.tv_sec = 0;
 			timeout.tv_usec = 10000;
 			fdmask = 1 << s;
 			if (select(s + 1, (fd_set *)&fdmask, (fd_set *)NULL,
-			    (fd_set *)NULL, &timeout) < 1) {
-				floodok = 1;
+				   (fd_set *)NULL, &timeout) < 1) {
+				got = 1;
 				continue;
 			}
 		}
@@ -524,7 +495,7 @@ main(int argc, char *argv[])
 			perror("ping: recvfrom");
 			continue;
 		}
-		pr_pack((char *)packet, cc, &from);
+		got = pr_pack((char *)packet, cc, &from);
 		if (npackets && nreceived >= npackets)
 			break;
 	}
@@ -544,22 +515,12 @@ main(int argc, char *argv[])
  * quality of the delay and loss statistics.
  */
 static void
-catcher(int signum)
+catcher(int ignore)
 {
-	struct tms buf;
-	clock_t current;
-	static clock_t last = 0;
 	int waittime;
 
-	if (signum) {
-		current = times(&buf);
-		if (current - last >= CLK_TCK - 1 || current < last) {
-			last = current;
-			pinger();
-		}
-	} else
-		pinger();
-
+	(void)ignore;
+	pinger();
 	(void)signal(SIGALRM, catcher);
 	if (!npackets || ntransmitted < npackets)
 		alarm((u_int)interval);
@@ -604,7 +565,10 @@ catcher(int signum)
  * will be added on by the kernel.  The ID field is our UNIX process ID,
  * and the sequence number is an ascending integer.  The first 8 bytes
  * of the data portion are used to hold a UNIX "timeval" struct in VAX
- * byte-order, to compute the round-trip time.
+ * byte-order, to compute the round-trip time. 
+ *
+ * Note: this may be 16 bytes now on some platforms - eventually all as
+ * time_t becomes 64-bit. And I think the VAX byte order thing is a lie.
  */
 static void
 pinger(void)
@@ -613,6 +577,23 @@ pinger(void)
 	register int cc;
 	int i;
 
+	/* probably the wrong place however, what we want to do here
+	 * is to check the last time we were called. if it's approx
+	 * a second, we proceed otherwise we return without doing anything
+	 * unless of course it's root calling the shot
+	 */
+
+	time_t this_time;
+
+	time(&this_time);
+
+#ifdef DEBUG
+	printf ("Time diff is %d seconds\n", this_time - last_time);
+#endif
+
+	if (((this_time - last_time) < 1) && (getuid() != 0))
+		return;
+
 	icp = (struct icmphdr *)outpack;
 	icp->icmp_type = ICMP_ECHO;
 	icp->icmp_code = 0;
@@ -622,22 +603,19 @@ pinger(void)
 
 	CLR(icp->icmp_seq % mx_dup_ck);
 
-	if (timing) {
+	if (timing)
 		(void)gettimeofday((struct timeval *)&outpack[8],
 		    (struct timezone *)NULL);
-#ifdef timestamp_check
-                timestamp[icp->icmp_seq % MAX_TIME_CHK] 
-			= *(struct timeval *)&outpack[8];
-#endif
-	}
 
 	cc = datalen + 8;			/* skips ICMP portion */
 
 	/* compute ICMP checksum here */
 	icp->icmp_cksum = in_cksum((u_short *)icp, cc);
 
-	i = sendto(s, (char *)outpack, cc, 0, &whereto,
-	    sizeof(struct sockaddr));
+	i = sendto(s, (char *)outpack, cc, 0, (struct sockaddr *)&whereto,
+	    sizeof(whereto));
+
+	time(&last_time);
 
 	if (i < 0 || i != cc)  {
 		if (i < 0)
@@ -655,8 +633,9 @@ pinger(void)
  * because ALL readers of the ICMP socket get a copy of ALL ICMP packets
  * which arrive ('tis only fair).  This permits multiple copies of this
  * program to be run without having intermingled output (or statistics!).
+ * Returns nonzero if it was one of our echo replies.
  */
-void
+int
 pr_pack(char *buf, int cc, struct sockaddr_in *from)
 {
 	register struct icmphdr *icp;
@@ -669,9 +648,10 @@ pr_pack(char *buf, int cc, struct sockaddr_in *from)
 	static char old_rr[MAX_IPOPTLEN];
 /*#endif*/
 	struct iphdr *ip;
-	struct timeval tv, *tp;
-	u_long triptime = 0;
-	int hlen, dupflag, negflag = 0;
+	struct timeval tv, packettv, *tp;
+	long triptime = 0;
+	int hlen, dupflag;
+	int rv;
 
 	(void)gettimeofday(&tv, (struct timezone *)NULL);
 
@@ -683,7 +663,7 @@ pr_pack(char *buf, int cc, struct sockaddr_in *from)
 			(void)fprintf(stderr,
 			  "ping: packet too short (%d bytes) from %s\n", cc,
 			  inet_ntoa(*(struct in_addr *)&from->sin_addr.s_addr));
-		return;
+		return 0;
 	}
 
 	/* Now the ICMP part */
@@ -691,45 +671,21 @@ pr_pack(char *buf, int cc, struct sockaddr_in *from)
 	icp = (struct icmphdr *)(buf + hlen);
 	if (icp->icmp_type == ICMP_ECHOREPLY) {
 		if (icp->icmp_id != ident)
-			return;			/* 'Twas not our ECHO */
+			return 0;		/* 'Twas not our ECHO */
 		++nreceived;
-		floodok = 1;
 		if (timing) {
 #ifndef icmp_data
 			tp = (struct timeval *)(icp + 1);
 #else
 			tp = (struct timeval *)icp->icmp_data;
 #endif
-#ifdef timestamp_check
-	                if ( icp->icmp_seq < ntransmitted
-			  && (ntransmitted - icp->icmp_seq <= MAX_TIME_CHK) 
-			  && !tvsame(timestamp[icp->icmp_seq % MAX_TIME_CHK], *tp ) ) 
-			{
-				(void)printf("\nwrong timestamp #%d should be %ds %dus but was %ds %dus\n", 
-				icp->icmp_seq, 
-				(int) timestamp[icp->icmp_seq % MAX_TIME_CHK].tv_sec,
-				(int) timestamp[icp->icmp_seq % MAX_TIME_CHK].tv_usec,
-				(int) tp->tv_sec, (int) tp->tv_usec );
-				tp = &timestamp[icp->icmp_seq % MAX_TIME_CHK];
-			}
-#endif
-
-#ifdef timestamp_check
-			if (tv.tv_sec < tp->tv_sec 
-			  || ( tv.tv_sec == tp->tv_sec && tv.tv_usec < tp->tv_usec ) ) {
-				(void)printf("\ntimestamp #%d earlier than current time %ds %dus; was %ds %dus\n", icp->icmp_seq, (int) tv.tv_sec, (int) tv.tv_usec, (int) tp->tv_sec, (int) tp->tv_usec );
-
-				negflag = 1;
-			} else {
-				negflag = 0;
-			}
-#endif
-			tvsub(&tv, tp);
+			memcpy(&packettv, tp, sizeof(struct timeval));
+			tvsub(&tv, &packettv);
 			triptime = tv.tv_sec * 10000 + (tv.tv_usec / 100);
 			tsum += triptime;
-			if (triptime < tmin && !negflag)
+			if (triptime < tmin)
 				tmin = triptime;
-			if (triptime > tmax && !negflag)
+			if (triptime > tmax)
 				tmax = triptime;
 		}
 
@@ -743,7 +699,9 @@ pr_pack(char *buf, int cc, struct sockaddr_in *from)
 		}
 
 		if (options & F_QUIET)
-			return;
+			return 1;
+
+		rv = 1;
 
 		if (options & F_FLOOD)
 			(void)write(STDOUT_FILENO, &BSPACE, 1);
@@ -753,21 +711,15 @@ pr_pack(char *buf, int cc, struct sockaddr_in *from)
 			   icp->icmp_seq);
 			(void)printf(" ttl=%d", ip->ip_ttl);
 			if (timing)
-				(void)printf(" time=%lu.%lu ms", triptime/10,
+				(void)printf(" time=%ld.%ld ms", triptime/10,
 						triptime%10);
 			if (dupflag)
 				(void)printf(" (DUP!)");
-
-#ifdef timestamp_check
-			if (negflag)
-				(void)printf(" (TIMETRAVEL!)");
-#endif
-
 			/* check the data */
 #ifndef icmp_data
-			cp = ((u_char*)(icp + 1) + 8);
+			cp = ((u_char*)(icp + 1) + sizeof(struct timeval));
 #else
-			cp = (u_char*)icp->icmp_data + 8;
+			cp = (u_char*)icp->icmp_data + sizeof(struct timeval);
 #endif
 			dp = &outpack[8 + sizeof(struct timeval)];
 			for (i = 8; i < datalen; ++i, ++cp, ++dp) {
@@ -787,10 +739,11 @@ pr_pack(char *buf, int cc, struct sockaddr_in *from)
 	} else {
 		/* We've got something other than an ECHOREPLY */
 		if (!(options & F_VERBOSE))
-			return;
+			return 0;
 		(void)printf("%d bytes from %s: ", cc,
 		    pr_addr(from->sin_addr.s_addr));
 		pr_icmph(icp);
+		rv = 0;
 	}
 
 /*#if 0*/
@@ -874,6 +827,8 @@ pr_pack(char *buf, int cc, struct sockaddr_in *from)
 		(void)putchar('\n');
 		(void)fflush(stdout);
 	}
+
+	return rv;
 }
 
 /*
@@ -900,6 +855,7 @@ in_cksum(u_short *addr, int len)
 
 	/* mop up an odd byte, if necessary */
 	if (nleft == 1) {
+		answer=0;
 		*(u_char *)(&answer) = *(u_char *)w ;
 		sum += answer;
 	}
@@ -943,16 +899,17 @@ finish(int ignore)
 	(void)printf("%ld packets received, ", nreceived);
 	if (nrepeats)
 		(void)printf("+%ld duplicates, ", nrepeats);
-	if (ntransmitted)
+	if (ntransmitted) {
 		if (nreceived > ntransmitted)
 			(void)printf("-- somebody's printing up packets!");
 		else
 			(void)printf("%d%% packet loss",
 			    (int) (((ntransmitted - nreceived) * 100) /
 			    ntransmitted));
+	}
 	(void)putchar('\n');
 	if (nreceived && timing)
-		(void)printf("round-trip min/avg/max = %lu.%lu/%lu.%lu/%lu.%lu ms\n",
+		(void)printf("round-trip min/avg/max = %ld.%ld/%lu.%ld/%ld.%ld ms\n",
 			tmin/10, tmin%10,
 			(tsum / (nreceived + nrepeats))/10,
 			(tsum / (nreceived + nrepeats))%10,
